from dataclasses import dataclass
from argparse import ArgumentParser
import os

from polyrepo.command import PolyRepoCommand
from polyrepo.gitlab import GitLab


@dataclass
class GroupsCommand(PolyRepoCommand):

    """Simplifying assumptions:

    - pwd maps to the GitLab host root
    - argument is the path to a directory that exists and maps to a GitLab
      group or subgroup"""

    path: str = ''
    traverse: bool = False
    name = 'groups'

    @classmethod
    def add_args(self, parser):
        parser.add_argument('path', nargs='?')
        parser.add_argument('--traverse', '-t', action='store_true')

    @PolyRepoCommand.wrap
    def execute(self):
        from pathlib import Path
        host = self.config_get('gitlab-host')
        token = self.config_get('gitlab-token')
        if host and token:
            subgroups = GitLab(host, token).group_subgroups(
                self.path, traverse=self.traverse)
            self.status = f"Listed {len(subgroups)} subgroups"
            return "\n".join(subgroups)
        else:
            raise RuntimeError("Missing host or token for GitLab API call")



from dataclasses import dataclass

from wizlib.command_handler import Command
from wizlib.config_machine import ConfigMachine


@dataclass
class PolyRepoCommand(ConfigMachine, Command):

    appname = 'polyrepo'
    default = 'sync'



import sys

from wizlib.command_handler import CommandHandler
from .command import PolyRepoCommand


class PolyRepoHandler(CommandHandler):

    @classmethod
    def shell(cls):
        super().shell(PolyRepoCommand)



from unittest import TestCase
import os

from wizlib.command_handler import CommandHandler
from polyrepo.command import PolyRepoCommand
from polyrepo.command.sync_command import SyncCommand


class TestCommandSync(TestCase):

    def test_from_handler(self):
        r, s = CommandHandler(PolyRepoCommand).handle(['sync'])
        self.assertEqual(r, 'Sync!')

    def test_default(self):
        r, s = CommandHandler(PolyRepoCommand).handle()
        self.assertEqual(r, 'Sync!')

    def test_argument(self):
        r, s = CommandHandler(PolyRepoCommand).handle(['sync', 'foo'])
        self.assertEqual(s, 'Synced foo.')

    def test_config(self):
        h = CommandHandler(PolyRepoCommand)
        c = h.get_command(
            ['--config', 'test/files/.polyrepo.yml', 'sync', 'foo'])
        self.assertEqual(c.config_get('gitlab-host'), 'gitlab.com')

    def test_dir_list(self):
        c = SyncCommand()
        c.gitlab_host = 'gitlab.local'
        self.assertEqual(c.config_get('gitlab-host'), 'gitlab.local')



from dataclasses import dataclass
from urllib.parse import quote
from json import dumps
from json import loads

from requests import post
from requests import codes


QUERY = """
query {{
  group(fullPath: "{path}") {{
    descendantGroups(after: "{endCursor}") {{
        nodes {{
            fullPath
        }}
        pageInfo {{
            endCursor
            hasNextPage
        }}
    }}
  }}
}}
"""


@dataclass
class GitLab:

    host: str = None
    token: str = None

    @property
    def headers(self):
        return {
            'Private-Token': self.token,
            'Content-Type': 'application/json'
        }

    def group_subgroups(self, path: str, traverse: bool = True):
        if not path:
            return
        url = f"https://{self.host}/api/graphql"
        paths = [path]
        hasNextPage = True
        endCursor = ""
        while hasNextPage:
            query = QUERY.format(**locals())
            response = post(url, headers=self.headers, json={'query': query})
            if response.status_code == codes.ok:
                groups = response.json()['data']['group']['descendantGroups']
                paths += [n['fullPath'] for n in groups['nodes']]
                hasNextPage = groups['pageInfo']['hasNextPage']
                if hasNextPage:
                    endCursor = groups['pageInfo']['endCursor']
            else:
                # Can we let requests take care of this?
                try:
                    message = response.json()['message']
                except Exception:
                    message = f"Error {response.status_code} from {url}"
                raise Exception(message)
        return sorted(paths)



from dataclasses import dataclass
from argparse import ArgumentParser

from polyrepo.command import PolyRepoCommand


@dataclass
class SyncCommand(PolyRepoCommand):

    """Simplifying assumptions:

    - pwd maps to the GitLab host root
    - argument is the path to a directory that exists and maps to a GitLab
      group or subgroup"""

    path: str = ''
    name = 'sync'

    @classmethod
    def add_args(self, parser):
        parser.add_argument('path', nargs='?')

    @PolyRepoCommand.wrap
    def execute(self):
        # self.host = self.cmachine.get('gitlab-host')
        self.status = f"Synced {self.path}."
        return "Sync!"
